function W = myKNN(X,nNeighbors,type,sigma)
% Inputs:
%   X: data matrix of size (dimension) x (number of samples)
%   nNeighbors: number of neighbors to connect for each point
%   type: weight type; 'constant' or 'gaussian'
%   sigma: tuning parameter for Gaussian weights
% Outputs:
%   W: weighted adjacency matrix

if nargin < 4
    sigma = 1;
end

[D,N] = size(X);
dmat = squareform(pdist(X'));

W = zeros(N);
for ii = 1:N
    di = dmat(ii,:);
    [vals,inds] = sort(di,'ascend');
    if strcmp(type,'constant')
        W(ii,inds(1:nNeighbors)) = 1;
        W(inds(1:nNeighbors),ii) = 1;
    else
        W(ii,inds(1:nNeighbors)) = exp(-vals(1:nNeighbors).^2/(2*sigma^2));
        W(inds(1:nNeighbors),ii) = exp(-vals(1:nNeighbors).^2/(2*sigma^2));
    end
end
W = W - diag(diag(W));
