"""
Spectral Clustering Demo - syntheticTest

ECE 510

python version: Python 3.7.2

Spring 2019
"""

import numpy as np
import matplotlib.pyplot as plt
from sklearn.cluster import KMeans
import helperFunctions as hf
from mySpectralClustering import mySpectralClustering

# Generate noisy data on two concentric spheres
D = 2
N = 400
K = 2
trueLabels = np.append(np.ones([int(N/2), 1]), 2 * np.ones([int(N/2), 1])).reshape(N,1)

var = 0.001

# cluster 1
X1 = np.random.randn(D, int(N/2))
X1 = X1 / np.tile(np.sqrt(sum(X1**2)), (D,1))
X1 = X1 + np.tile(np.sqrt(var) * np.random.randn(1, int(N/2)), (D,1))

# cluster 2
X2 = np.random.randn(D, int(N/2))
X2 = 2*X2 / np.tile(np.sqrt(sum(X2**2)), (D,1))
X2 = X2 + np.tile(np.sqrt(var) * np.random.randn(1, int(N/2)), (D,1))

X = np.append(X1, X2, axis=1)

# Data visualization
plt.figure(figsize=(6,6))
plt.scatter(X[0, 0:int(N/2)], X[1, 0:int(N/2)], marker='X')
plt.scatter(X[0, int(N/2):N], X[1, int(N/2):N], marker='X')
plt.title("Sythetic Data - True Clusters")
plt.show()

# Kmeans
k_means = KMeans(n_clusters=K, max_iter=100).fit(X.T)
estLabels = k_means.labels_.reshape(N,1)
kmeanserr, estLabels = hf.missRate(trueLabels, estLabels)
print(f"KMeans error:{kmeanserr}")

plt.figure(figsize=(6,6))
plt.scatter(X[0, np.argwhere(estLabels==1)], X[1, np.argwhere(estLabels==1)], marker='X')
plt.scatter(X[0, np.argwhere(estLabels==2)], X[1, np.argwhere(estLabels==2)], marker='X')
plt.title("Sythetic Data - K-Means Clusters")
plt.show()

# Spectral Clustering
# build adjacency matrix
nNeighbors = 20
wtype = 'constant'
sigma = 1
W = hf.myKNN(X, nNeighbors, wtype, sigma)

# Look at the resulting graph: 
plt.figure(figsize=(8,8))
plt.imshow(W)
plt.show()

# apply spectral clustering
estLabels, Y = mySpectralClustering(W, K, 0)
scErrUn, UnestLabels_SP = hf.missRate(trueLabels, estLabels)
print(f"Spectral Clustering error (unnormalized): {scErrUn}")

estLabels, Y = mySpectralClustering(W, K, 1)
scErr, estLabels_SP = hf.missRate(trueLabels, estLabels)
print(f"Spectral Clustering error (normalized): {scErr}")

# look at the data in the transformed space
plt.figure(figsize=(6,6))
plt.scatter(Y[0, 0:int(N/2)], Y[1, 0:int(N/2)], marker='X')
plt.scatter(Y[0, int(N/2):N], Y[1, int(N/2):N], marker='X')
plt.title("Transformed Data")
plt.show()
